//	K ENB  
//	Multiple Passes Lenzes w_Individual Size and Color Control 
//	Code extensive modifications by KYO aka Oyama w/some help from Boris ;)
//	Lenz Texture by OpethFeldt
//	Copyright (c) 2007-2013 Boris Vorontsov
//	Medium Hexagons

//=====================================================================================================
//	Internal parameters
//=====================================================================================================

float ELenzIntensity=0.805;	//[Global Intensity]
float LenzScaleFactor=0.288;	//[Global Scale, will affect all lenzes]	//1.0, 0.75, 0.66, 0.5, 0.25, 0.33, 0.322, so on

float LenzFactorDraw=1.0;	//[Allows for separated factors in case of additional Draw techniques, such as Draw2, ect...]

float LenzContrastDraw=1.3;	//[Contrast to clouds - More is weaker opacity compared to clouds]

/**
 *	Separated Lenzes controls for Draw techniques : Offset, Scale, Scale Factor, Color RGB, Color Multiplier
 *	KYO : color balance will also act as a separate intensity control for each lenz 
 *	Color : 1.0, 1.0, 1.0 is full intensity / opacity
 *	If color components are below 1.0, lenz is losing opacity
 *	You can counterbalance it by increasing f#_ColorMultiplier over 1.0
 *	Fine tuning is balancing global intensity, RGB balance, multiplier
 */

//Lenz #1
#define f1_LenzOffset		-0.8				//Position from Sun to Eye	
#define f1_LenzScale		0.083				//Size of the Lenz Flare
#define f1_LenzFactor		1.0				//Multiplier for Size, in case 2 lenzes are sharing close offset a/o scale	
#define	f1_LenzColor		float3(0.75, 0.75, 0.50)	//Color in RGB (0.00 to 1.00) for the Lenz Flare	
#define	f1_ColorMultiplier	1.0				//Color Multiplier, in case RGB values are too low - Fine Tuning

//Lenz #2
#define f2_LenzOffset		-0.47	
#define f2_LenzScale		0.166
#define f2_LenzFactor		1.0	
#define	f2_LenzColor		float3(0.85, 0.65, 0.85)	
#define	f2_ColorMultiplier	1.0

//Lenz #3
#define f3_LenzOffset		-0.14	
#define f3_LenzScale		0.212
#define f3_LenzFactor		1.0
#define	f3_LenzColor		float3(1.0, 1.0, 1.0)	
#define	f3_ColorMultiplier	1.0

//Lenz #4
#define f4_LenzOffset		0.14	
#define f4_LenzScale		0.249
#define f4_LenzFactor		0.75	
#define	f4_LenzColor		float3(0.25, 1.0, 0.75)	
#define	f4_ColorMultiplier	1.0

//Lenz #5
#define f5_LenzOffset		0.47	
#define f5_LenzScale		0.332
#define f5_LenzFactor		1.0	
#define	f5_LenzColor		float3(0.0, 0.0, 0.85)
#define	f5_ColorMultiplier	2.0

//Lenz #6
#define f6_LenzOffset		0.8	
#define f6_LenzScale		0.415
#define f6_LenzFactor		0.75	
#define	f6_LenzColor		float3(0.90, 0.30, 0.75)	
#define	f6_ColorMultiplier	1.0

//Lenz #7
#define f7_LenzOffset		1.13	
#define f7_LenzScale		0.498
#define f7_LenzFactor		1.0	
#define	f7_LenzColor		float3(1.0, 1.0, 1.0)
#define	f7_ColorMultiplier	1.0

//Lenz #8
#define f8_LenzOffset		1.46	
#define f8_LenzScale		0.581
#define f8_LenzFactor		0.5	
#define	f8_LenzColor		float3(0.25, 0.95, 0.45)	
#define	f8_ColorMultiplier	1.5

//Lenz #9
#define f9_LenzOffset		1.79	
#define f9_LenzScale		0.581
#define f9_LenzFactor		0.85	
#define	f9_LenzColor		float3(1.0, 1.0, 1.0)	
#define	f9_ColorMultiplier	1.0

//Lenz #10
#define f10_LenzOffset		2.12	
#define f10_LenzScale		0.581
#define f10_LenzFactor		1.0	
#define	f10_LenzColor		float3(0.95, 0.55, 0.85)	
#define	f10_ColorMultiplier	1.0

//Lenz #11
#define f11_LenzOffset		2.43	
#define f11_LenzScale		0.664
#define f11_LenzFactor		1.11	
#define	f11_LenzColor		float3(0.75, 0.75, 0.42)
#define	f11_ColorMultiplier	1.0


//Add or Remove additional Lenzes controls here, as done above
//This being done, a/o remove passes of technique below

//=====================================================================================================
//	External parameters
//=====================================================================================================
//Keyboard controlled temporary variables (in some versions exists in the config file). 
//Press and hold key 1,2,3...8 together with PageUp or PageDown to modify. By default all set to 1.0
float4	tempF1; //0,1,2,3
float4	tempF2; //5,6,7,8
float4	tempF3; //9,0
//x=Width, y=1/Width, z=ScreenScaleY, w=1/ScreenScaleY
float4	ScreenSize;
//x=generic timer in range 0..1, period of 16777216 ms (4.6 hours), w=frame time elapsed (in seconds)
float4	Timer;
//xy=sun position on screen, w=visibility
float4	LightParameters;

//textures
texture2D texColor;
texture2D texMask;

sampler2D SamplerColor = sampler_state
{
	Texture   = <texColor>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;				
	MipMapLodBias=0;
};

sampler2D SamplerMask = sampler_state
{
	Texture   = <texMask>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

struct VS_OUTPUT_POST
{
	float4 vpos  : POSITION;
	float2 txcoord : TEXCOORD0;
};

struct VS_INPUT_POST
{
	float3 pos  : POSITION;
	float2 txcoord : TEXCOORD0;
};

/**
 * Several sprites moving similar to lenz effect
 * They are transformed in vertex shader and drawed separately for better performance
 * Offset is set in passes of technique
 */

VS_OUTPUT_POST VS_Draw(VS_INPUT_POST IN, uniform float offset, uniform float scale)
{
	VS_OUTPUT_POST OUT;

	float4 pos=float4(IN.pos.x,IN.pos.y,IN.pos.z,1.0);
	pos.y*=ScreenSize.z;

	//create own parameters instead of this, including uv offsets
	float2 shift=LightParameters.xy * offset;
	pos.xy=pos.xy * (scale * LenzScaleFactor) - shift;

	OUT.vpos=pos;
	OUT.txcoord.xy=IN.txcoord.xy;

	return OUT;
}

float4 PS_Draw(VS_OUTPUT_POST IN, float2 vPos : VPOS, uniform float3 colorfilter, uniform float colormultiplier) : COLOR
{
	float4 res;
	float2 coord=IN.txcoord.xy;

	//read sun visibility as amount of effect
	float sunmask=tex2D(SamplerMask, float2(0.5, 0.5)).x;
	sunmask=pow(sunmask, LenzContrastDraw);						//more contrast to clouds	
	clip(sunmask-0.02);//early exit if too low

	float4 origcolor=tex2D(SamplerColor, coord.xy);
	sunmask*=LightParameters.w * (ELenzIntensity * LenzFactorDraw);
	res.xyz=origcolor * sunmask;

	float clipper=dot(res.xyz, 0.333);
	clip(clipper-0.0003);//skip draw if black

	res.xyz*=colorfilter * colormultiplier;
	res.w=1.0;
	return res;
}

//-------------------------------------------------------------------------------------------------------------------------

/**
 *	TECHNIQUES
 */

technique Draw
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_Draw(f1_LenzOffset, (f1_LenzScale * f1_LenzFactor));	//offset, scale, scale factor
		PixelShader  = compile ps_3_0 PS_Draw(f1_LenzColor, f1_ColorMultiplier);		//Color RGB

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P1
	{
		VertexShader = compile vs_3_0 VS_Draw(f2_LenzOffset, (f2_LenzScale * f2_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f2_LenzColor, f2_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P2
	{
		VertexShader = compile vs_3_0 VS_Draw(f3_LenzOffset, (f3_LenzScale * f3_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f3_LenzColor, f3_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P3
	{
		VertexShader = compile vs_3_0 VS_Draw(f4_LenzOffset, (f4_LenzScale * f4_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f4_LenzColor, f4_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P4
	{
		VertexShader = compile vs_3_0 VS_Draw(f5_LenzOffset, (f5_LenzScale * f5_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f5_LenzColor, f5_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P5
	{
		VertexShader = compile vs_3_0 VS_Draw(f6_LenzOffset, (f6_LenzScale * f6_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f6_LenzColor, f6_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P6
	{
		VertexShader = compile vs_3_0 VS_Draw(f7_LenzOffset, (f7_LenzScale * f7_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f7_LenzColor, f7_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P7
	{
		VertexShader = compile vs_3_0 VS_Draw(f8_LenzOffset, (f8_LenzScale * f8_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f8_LenzColor, f8_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P8
	{
		VertexShader = compile vs_3_0 VS_Draw(f9_LenzOffset, (f9_LenzScale * f9_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f9_LenzColor, f9_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P9
	{
		VertexShader = compile vs_3_0 VS_Draw(f10_LenzOffset, (f10_LenzScale * f10_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f10_LenzColor, f10_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}

	pass P10
	{
		VertexShader = compile vs_3_0 VS_Draw(f11_LenzOffset, (f11_LenzScale * f11_LenzFactor));
		PixelShader  = compile ps_3_0 PS_Draw(f11_LenzColor, f11_ColorMultiplier);

		AlphaBlendEnable=TRUE;
		SrcBlend=ONE;
		DestBlend=ONE;

		ColorWriteEnable = ALPHA|RED|GREEN|BLUE;
		DitherEnable=FALSE;
		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		StencilEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}
}
// KYO : and so on...